@@ -70,6 +70,11 @@
 /*	IBM T.J. Watson Research
 /*	P.O. Box 704
 /*	Yorktown Heights, NY 10598, USA
+/*
+/*	Wietse Venema
+/*	Google, Inc.
+/*	111 8th Avenue
+/*	New York, NY 10011, USA
 /*--*/
 
 /* System library. */
@@ -107,6 +112,7 @@
 #include <abounce.h>
 #include <rec_type.h>
 #include <qmgr_user.h>
+#include <info_log_addr_form.h>
 
 /* Application-specific. */
 
@@ -115,12 +121,12 @@
  /*
   * A bunch of call-back routines.
   */
-static void qmgr_active_done_2_bounce_flush(int, char *);
+static void qmgr_active_done_2_bounce_flush(int, void *);
 static void qmgr_active_done_2_generic(QMGR_MESSAGE *);
-static void qmgr_active_done_25_trace_flush(int, char *);
+static void qmgr_active_done_25_trace_flush(int, void *);
 static void qmgr_active_done_25_generic(QMGR_MESSAGE *);
-static void qmgr_active_done_3_defer_flush(int, char *);
-static void qmgr_active_done_3_defer_warn(int, char *);
+static void qmgr_active_done_3_defer_flush(int, void *);
+static void qmgr_active_done_3_defer_warn(int, void *);
 static void qmgr_active_done_3_generic(QMGR_MESSAGE *);
 
 /* qmgr_active_corrupt - move corrupted file out of the way */
@@ -280,7 +286,7 @@ void    qmgr_active_done(QMGR_MESSAGE *message)
      * 
      * Bounces are sent asynchronously to avoid stalling while the cleanup
      * daemon waits for the qmgr to accept the "new mail" trigger.
-     *
+     * 
      * See also code in cleanup_bounce.c.
      */
     if (stat(mail_queue_path((VSTRING *) 0, MAIL_QUEUE_BOUNCE, message->queue_id), &st) == 0) {
@@ -301,7 +307,7 @@ void    qmgr_active_done(QMGR_MESSAGE *message)
 			      message->dsn_envid,
 			      message->dsn_ret,
 			      qmgr_active_done_2_bounce_flush,
-			      (char *) message);
+			      (void *) message);
 	    else
 		abounce_flush_verp(BOUNCE_FLAG_KEEP,
 				   message->queue_name,
@@ -313,7 +319,7 @@ void    qmgr_active_done(QMGR_MESSAGE *message)
 				   message->dsn_ret,
 				   message->verp_delims,
 				   qmgr_active_done_2_bounce_flush,
-				   (char *) message);
+				   (void *) message);
 	    return;
 	}
     }
@@ -326,7 +332,7 @@ void    qmgr_active_done(QMGR_MESSAGE *message)
 
 /* qmgr_active_done_2_bounce_flush - process abounce_flush() status */
 
-static void qmgr_active_done_2_bounce_flush(int status, char *context)
+static void qmgr_active_done_2_bounce_flush(int status, void *context)
 {
     QMGR_MESSAGE *message = (QMGR_MESSAGE *) context;
 
@@ -385,7 +391,8 @@ static void qmgr_active_done_2_generic(QMGR_MESSAGE *message)
      * 
      * See also comments in bounce/bounce_notify_util.c.
      */
-    if ((message->tflags & (DEL_REQ_FLAG_USR_VRFY | DEL_REQ_FLAG_RECORD))
+    if ((message->tflags & (DEL_REQ_FLAG_USR_VRFY | DEL_REQ_FLAG_RECORD
+			    | DEL_REQ_FLAG_REC_DLY_SENT))
 	|| (message->rflags & QMGR_READ_FLAG_NOTIFY_SUCCESS)) {
 	atrace_flush(message->tflags,
 		     message->queue_name,
@@ -396,7 +403,7 @@ static void qmgr_active_done_2_generic(QMGR_MESSAGE *message)
 		     message->dsn_envid,
 		     message->dsn_ret,
 		     qmgr_active_done_25_trace_flush,
-		     (char *) message);
+		     (void *) message);
 	return;
     }
 
@@ -408,7 +415,7 @@ static void qmgr_active_done_2_generic(QMGR_MESSAGE *message)
 
 /* qmgr_active_done_25_trace_flush - continue after atrace_flush() completion */
 
-static void qmgr_active_done_25_trace_flush(int status, char *context)
+static void qmgr_active_done_25_trace_flush(int status, void *context)
 {
     QMGR_MESSAGE *message = (QMGR_MESSAGE *) context;
 
@@ -438,7 +445,7 @@ static void qmgr_active_done_25_generic(QMGR_MESSAGE *message)
 	if (event_time() >= message->create_time +
 	    (*message->sender ? var_max_queue_time : var_dsn_queue_time)) {
 	    msg_info("%s: from=<%s>, status=expired, returned to sender",
-		     message->queue_id, message->sender);
+	     message->queue_id, info_log_addr_form_sender(message->sender));
 	    if (message->verp_delims == 0 || var_verp_bounce_off)
 		adefer_flush(BOUNCE_FLAG_KEEP,
 			     message->queue_name,
@@ -449,7 +456,7 @@ static void qmgr_active_done_25_generic(QMGR_MESSAGE *message)
 			     message->dsn_envid,
 			     message->dsn_ret,
 			     qmgr_active_done_3_defer_flush,
-			     (char *) message);
+			     (void *) message);
 	    else
 		adefer_flush_verp(BOUNCE_FLAG_KEEP,
 				  message->queue_name,
@@ -461,7 +468,7 @@ static void qmgr_active_done_25_generic(QMGR_MESSAGE *message)
 				  message->dsn_ret,
 				  message->verp_delims,
 				  qmgr_active_done_3_defer_flush,
-				  (char *) message);
+				  (void *) message);
 	    return;
 	} else if (message->warn_time > 0
 		   && event_time() >= message->warn_time - 1) {
@@ -476,7 +483,7 @@ static void qmgr_active_done_25_generic(QMGR_MESSAGE *message)
 			message->dsn_envid,
 			message->dsn_ret,
 			qmgr_active_done_3_defer_warn,
-			(char *) message);
+			(void *) message);
 	    return;
 	}
     }
@@ -489,7 +496,7 @@ static void qmgr_active_done_25_generic(QMGR_MESSAGE *message)
 
 /* qmgr_active_done_3_defer_warn - continue after adefer_warn() completion */
 
-static void qmgr_active_done_3_defer_warn(int status, char *context)
+static void qmgr_active_done_3_defer_warn(int status, void *context)
 {
     QMGR_MESSAGE *message = (QMGR_MESSAGE *) context;
 
@@ -503,7 +510,7 @@ static void qmgr_active_done_3_defer_warn(int status, char *context)
 
 /* qmgr_active_done_3_defer_flush - continue after adefer_flush() completion */
 
-static void qmgr_active_done_3_defer_flush(int status, char *context)
+static void qmgr_active_done_3_defer_flush(int status, void *context)
 {
     QMGR_MESSAGE *message = (QMGR_MESSAGE *) context;
 
